package Lingua::Pere::Dict;

use locale;
use strict;

use vars qw(@ISA $VERSION);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);

use POSIX;
use DB_File;
use Lingua::Pere::Misc::Text qw(
	enctext
	dectext
);

#**************************************************************************
sub new {
	my ($class, $opts) = @_;

	my $self = bless {
		wdir => (exists($opts->{wdir}) && defined($opts->{wdir}) && ($opts->{wdir} ne '')) ? $opts->{wdir} : '.',
		mode => $opts->{mode} || O_RDONLY,
		psiz => $opts->{psiz} || undef,
		csiz => $opts->{csiz} || undef,
		flgs => $opts->{flgs} || undef,
		comp => $opts->{comp} || undef,
		nams => {},
	}, $class;

	return $self;
}

#**************************************************************************
sub DESTROY {
	my ($self) = @_;
	
	$self->close if (defined $self);
}

#**************************************************************************
sub open {
	my ($self, $name, $file, $opts) = @_;
	
	return undef unless ($name);
	$file ||= $name;

	unless (exists $self->{nams}->{$name}) {
		my $dbit = new DB_File::BTREEINFO;

		if (defined($opts) && exists($opts->{psiz}) && defined($opts->{psiz}) && ($opts->{psiz} > 0)) {
			$dbit->{psize} = $opts->{psiz};
		} elsif (defined($self->{psiz}) && ($self->{psiz} > 0)) {
			$dbit->{psize} = $self->{psiz};
		}

		if (defined($opts) && exists($opts->{csiz}) && defined($opts->{csiz}) && ($opts->{csiz} > 0)) {
			$dbit->{cachesize} = $opts->{csiz};
		} elsif (defined($self->{csiz}) && ($self->{csiz} > 0)) {
			$dbit->{cachesize} = $self->{csiz};
		}

		if (defined($opts) && exists($opts->{flgs}) && defined($opts->{flgs})) {
			$dbit->{flags} = $opts->{flgs};
		} elsif (defined($self->{flgs})) {
			$dbit->{flags} = $self->{flgs};
		}

		if (defined($opts) && exists($opts->{comp}) && defined($opts->{comp})) {
			$dbit->{compare} = $opts->{comp};
		} elsif (defined($self->{comp})) {
			$dbit->{compare} = $self->{comp};
		}

		my $wdir = (exists($opts->{wdir}) && defined($opts->{wdir})) ? $opts->{wdir} : $self->{wdir};
		my $mode = (exists($opts->{mode}) && defined($opts->{mode})) ? $opts->{mode} : $self->{mode};

		my %h;
		my $d;

		$d = tie(%h, 'DB_File', sprintf("%s/%s",$wdir,$file), $mode, 0644, $dbit) or die "Can not open '$wdir/$file': $!";

		$self->{nams}->{$name} = {h => \%h, d => $d, f => $file};
	}
	
	return $self->{nams}->{$name};
}

#**************************************************************************
sub geth {
	my ($self, $name, $file, $opts) = @_;
	
	my $h = $self->open($name, $file, $opts);
	
	return ($h) ? $h->{h} : undef;
}

#**************************************************************************
sub getd {
	my ($self, $name, $file, $opts) = @_;
	
	my $h = $self->open($name, $file, $opts);
	
	return ($h) ? $h->{d}: undef;
}

#**************************************************************************
sub sync {
	my ($self, $name) = @_;
	
	my $nams = $self->{nams};
	if ($name) {
		if (ref($name) eq 'ARRAY') {
			foreach my $xnam (@{$name}) {
				if (exists $nams->{$xnam}) {
					my $d = $nams->{$xnam}->{d};
					$d->sync if ($d);
				}
			}
		} else {
			if (exists $nams->{$name}) {
				my $d = $nams->{$name}->{d};
				$d->sync if ($d);
			}
		}
	} else {
		foreach my $xnam (keys %{$nams}) {
			my $d = $nams->{$xnam}->{d};
			$d->sync if ($d);
		}
	}
}

#**************************************************************************
sub close {
	my ($self, $name) = @_;
	
	$self->sync($name);
	
	my $nams = $self->{nams};
	if ($name) {
		if (ref($name) eq 'ARRAY') {
			foreach my $xnam (@{$name}) {
				if (exists $nams->{$xnam}) {
					undef($nams->{$xnam}->{d}) if (defined $nams->{$xnam}->{d});
				}
			}

			foreach my $xnam (@{$name}) {
				if (exists $nams->{$xnam}) {
					my $h = $nams->{$xnam};
					if (defined $h) {
						untie($h->{h}) if (tied $h->{h});
						undef($h);
					}
					delete($nams->{$xnam});
				}
			}
		} else {
			if (exists $nams->{$name}) {
				my $h = $nams->{$name};
				if (defined $h) {
					undef($h->{d}) if (defined $h->{d});
					untie($h->{h}) if (defined($h->{d}) && tied($h->{h}));
					undef($h);
				}
				delete($nams->{$name});
			}
		}
	} else {
		foreach my $xnam (keys %{$nams}) {
			undef($nams->{$xnam}->{d}) if (defined $nams->{$xnam}->{d});
		}

		foreach my $xnam (keys %{$nams}) {
			my $h = $nams->{$xnam};
			if (defined $h) {
				untie($h->{h}) if (tied $h->{h});
				undef($h);
			}
			delete($nams->{$xnam});
		}
	}
}

#**************************************************************************
sub kill {
	my ($self, $name) = @_;
	
	$self->close($name);
	
	my $nams = $self->{nams};
	if ($name) {
		if (ref($name) eq 'ARRAY') {
			foreach my $xnam (@{$name}) {
				unlink(sprintf("%s/%s",$self->{wdir},$xnam));
			}
		} else {
			unlink(sprintf("%s/%s",$self->{wdir},$name));
		}
	} else {
#
	}
}

#**************************************************************************
sub clear {
	my ($self, $name) = @_;

	my $nams = $self->{nams};
	if ($name) {
		if (ref($name) eq 'ARRAY') {
			foreach my $xnam (@{$name}) {
				if (exists $nams->{$xnam}) {
					%{$nams->{$xnam}->{h}} = () if ($nams->{$xnam}->{h});
				}
			}
		} else {
			if (exists $nams->{$name}) {
				%{$nams->{$name}->{h}} = () if ($nams->{$name}->{h});
			}
		}
	} else {
		foreach my $xnam (keys %{$nams}) {
			%{$nams->{$xnam}->{h}} = () if ($nams->{$xnam}->{h});
		}
	}

	$self->sync($name);
}

#**************************************************************************
sub encwords {
	my ($self, $wa) = @_;

	my $hwdsf = $self->geth("wdsf");
	my $hwdsb = $self->geth("wdsb");
	my $hconf = $self->geth("conf");

	my $res = [];
	my $scn =	$hconf->{'scn'} || 1024;
	foreach my $wd (@{$wa}) {
		next if ($wd eq '');

		$wd = dectext($wd);
		next unless (defined($wd) && ($wd ne ''));

		if (exists $hwdsf->{$wd}) {
			push(@{$res},$hwdsf->{$wd});
		} else {
			my $v = pack('l',$scn++);
			$hwdsf->{$wd} = $v;
			$hwdsb->{$v} = $wd;
			push(@{$res},$v);
		}
	}
	$hconf->{'scn'} = $scn;
	
	return $res;
}

#**************************************************************************
sub decwords {
	my ($self, $ca) = @_;

	my $hwdsb = $self->geth("wdsb");

	foreach my $cd (@{$ca}) {
		if (defined($cd) && ($cd ne '')) {
			$cd = pack('l', $cd);
			if (exists $hwdsb->{$cd}) {
				$cd = $hwdsb->{$cd};
			} else {
				$cd = '_???_';
			}
		} else {
			$cd = '';
		}
		$cd = enctext($cd);
	}
	
	return $ca;
}

#**************************************************************************
sub encseq {
	my ($self, $sq) = @_;

	my $hconf = $self->geth("conf");
	my $hseqf = $self->geth('seqf');
	my $hseqb = $self->geth('seqb');

	my $cd = $sq->[0];
	if (scalar @{$sq} > 1) {
		my $scn =	$hconf->{'scn'} || 1024;
		for (my $i = 1; $i <= $#{$sq}; $i++) {
			my $r = $sq->[$i];
			my $b = $cd.$r;
			if (exists $hseqf->{$b}) {
				$cd = $hseqf->{$b};
			} else {
				my $v = pack('l',$scn++);
				$hseqf->{$b} = $v;
				$hseqb->{$v} = $b;
				$cd = $v;
			}
		}
		$hconf->{'scn'} = $scn;
	}
	
	return $cd;
}

#**************************************************************************
sub fndseq {
	my ($self, $sq) = @_;

	my $hseqf = $self->geth('seqf');
	my $hseqb = $self->geth('seqb');

	my $cd = $sq->[0];
	if (scalar @{$sq} > 1) {
		for (my $i = 1; $i <= $#{$sq}; $i++) {
			my $r = $sq->[$i];
			my $b = $cd.$r;
			if (exists $hseqf->{$b}) {
				$cd = $hseqf->{$b};
			} else {
				$cd = undef;
				last;
			}
		}
	}
	
	return $cd;
}

#**************************************************************************
sub decseq {
	my ($self, $cd, $sq) = @_;
	$sq ||= [];

	my $hseqf = $self->geth('seqf');
	my $hseqb = $self->geth('seqb');

	my @a = ();
	while (exists $hseqb->{$cd}) {
		my ($l, $r) = unpack('ll', $hseqb->{$cd});
		unshift(@a, $r);
		$cd = pack('l',$l);
	}
	$cd = unpack('l', $cd);
	unshift(@a, $cd);
	push(@{$sq}, @a);
	
	return $sq;
}

#**************************************************************************
sub decseqa {
	my ($self, $ac) = @_;
	my $sq = [];
	foreach my $cd (@{$ac}) {
		$self->decseq($cd, $sq);
	}
	
	return $sq;
}

#**************************************************************************
return 1;
__END__
