package Lingua::Pere::Misc::File;

require 5.000;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT_OK);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);

@EXPORT_OK = qw(
	openfile
	readfile
	writefile
	makedir
	textdir
);

use POSIX;
use File::Spec;
use Carp qw(
	carp
	croak
);
use IO::File;

#**************************************************************************
sub openfile {
	my $fil = shift;

	my $fh;
	my $st = 1;
	if (ref($fil)) {
		if (UNIVERSAL::isa( $fil, 'IO::Scalar')) {
			$fh = $fil;
		} elsif (UNIVERSAL::isa($fil, 'IO::Handle')	or UNIVERSAL::isa($fil, 'GLOB')) {
			$fh = $fil;
		} else {
			$fh = IO::File->new;
			$st = $fh->fdopen($fil, @_);
		}
	} else {
		$fh = IO::File->new;
		$st = $fh->open($fil, @_);
	}

	if ($st) {
		if ($fh->can('binmode')) {
			$fh->binmode;
		} else {
			binmode($fh);
		}
	} else {
		$fh = undef;
		carp("Can not open file '$fil': $!");
	}

	return $fh;
}

#**************************************************************************
sub readfile {
	my $fil = shift;

	my $fh = openfile($fil, (scalar(@_) > 0) ? @_ : 'r');

	my $bin = '';
	if (defined($fh)) {
		local $/ = undef;
		$bin = <$fh>;
		$fh->close;
		$/ = "\n";
	} else {
		$bin = undef;
	}

	return $bin;
}

#**************************************************************************
sub writefile {
	my $fil = shift;
	my $bin = shift;

	my $fh = openfile($fil, (scalar(@_) > 0) ? @_ : 'w');

	if (defined($fh)) {
		$fh->print($bin);
		$fh->close;
	}
}

#**************************************************************************
sub makedir {
	my ($dat, $dir) = @_;

	if (defined($dir) && ($dir ne '')) {
		my @a = grep { $_ ne '' } split(/\//, $dat);
		foreach my $d (@a) {
			$dat .= "/$d";
			mkdir($dat, 0755);
		}
	} else {
		my @a = split(/\//, $dat);
		$dat = shift(@a);
		$dat = "/" if ($dat eq '');
		mkdir($dat, 0755);
		foreach my $d (@a) {
			$dat .= "/$d";
			mkdir($dat, 0755);
		}
	}
	
	return $dat;
}

#**************************************************************************
sub textdir {
	my ($dir, $end) = @_;
	$end ||= '';

	opendir(DIR, $dir) or croak("Can not open directory '$dir': $!");
	my @con = readdir DIR;
	closedir(DIR);
	
	my @a;
	
	if ($end eq '') {
		@a = sort (grep { $_ !~ /^[.]{1,2}$/} @con);
	} else {
		@a = sort (grep {/^.+\.$end$/i} @con);
	}
 
	return wantarray ? @a : \@a;
}

#**************************************************************************
return 1;
