package Lingua::Pere::Misc::HTML;

require 5.000;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);
@EXPORT = ();
@EXPORT_OK = qw(
	ishtml
	encuri
	decuri
	decentis
	encentis
);

use POSIX;
use Lingua::Pere::Misc::Text qw(
	enctext
	dectext
);

my %ENTIS = (
	"&quot;"     => '"',   # &#0034; &#x0022;
	"&amp;"      => "&",   # &#0038; &#x0026;
	"&apos;"     => "'",   # &#0039; &#x0027;
	"&lt;"       => "<",   # &#0060; &#x003c;
	"&gt;"       => ">",   # &#0062; &#x003e;
	"&nbsp;"     => " ",  # &#0160; &#x00a0;
	"&iexcl;"    => "¡",  # &#0161; &#x00a1;
	"&cent;"     => "¢",  # &#0162; &#x00a2;
	"&pound;"    => "£",  # &#0163; &#x00a3;
	"&curren;"   => "¤",  # &#0164; &#x00a4;
	"&yen;"      => "¥",  # &#0165; &#x00a5;
	"&brvbar;"   => "¦",  # &#0166; &#x00a6;
	"&sect;"     => "§",  # &#0167; &#x00a7;
	"&uml;"      => "¨",  # &#0168; &#x00a8;
	"&copy;"     => "©",  # &#0169; &#x00a9;
	"&ordf;"     => "ª",  # &#0170; &#x00aa;
	"&laquo;"    => "«",  # &#0171; &#x00ab;
	"&not;"      => "¬",  # &#0172; &#x00ac;
	"&shy;"      => "­",  # &#0173; &#x00ad;
	"&reg;"      => "®",  # &#0174; &#x00ae;
	"&macr;"     => "¯",  # &#0175; &#x00af;
	"&deg;"      => "°",  # &#0176; &#x00b0;
	"&plusmn;"   => "±",  # &#0177; &#x00b1;
	"&sup2;"     => "²",  # &#0178; &#x00b2;
	"&sup3;"     => "³",  # &#0179; &#x00b3;
	"&acute;"    => "´",  # &#0180; &#x00b4;
	"&micro;"    => "µ",  # &#0181; &#x00b5;
	"&para;"     => "¶",  # &#0182; &#x00b6;
	"&middot;"   => "·",  # &#0183; &#x00b7;
	"&cedil;"    => "¸",  # &#0184; &#x00b8;
	"&sup1;"     => "¹",  # &#0185; &#x00b9;
	"&ordm;"     => "º",  # &#0186; &#x00ba;
	"&raquo;"    => "»",  # &#0187; &#x00bb;
	"&frac14;"   => "¼",  # &#0188; &#x00bc;
	"&frac12;"   => "½",  # &#0189; &#x00bd;
	"&frac34;"   => "¾",  # &#0190; &#x00be;
	"&iquest;"   => "¿",  # &#0191; &#x00bf;
	"&Agrave;"   => "À",  # &#0192; &#x00c0;
	"&Aacute;"   => "Á",  # &#0193; &#x00c1;
	"&Acirc;"    => "Â",  # &#0194; &#x00c2;
	"&Atilde;"   => "Ã",  # &#0195; &#x00c3;
	"&Auml;"     => "Ä",  # &#0196; &#x00c4;
	"&Aring;"    => "Å",  # &#0197; &#x00c5;
	"&AElig;"    => "Æ",  # &#0198; &#x00c6;
	"&Ccedil;"   => "Ç",  # &#0199; &#x00c7;
	"&Egrave;"   => "È",  # &#0200; &#x00c8;
	"&Eacute;"   => "É",  # &#0201; &#x00c9;
	"&Ecirc;"    => "Ê",  # &#0202; &#x00ca;
	"&Euml;"     => "Ë",  # &#0203; &#x00cb;
	"&Igrave;"   => "Ì",  # &#0204; &#x00cc;
	"&Iacute;"   => "Í",  # &#0205; &#x00cd;
	"&Icirc;"    => "Î",  # &#0206; &#x00ce;
	"&Iuml;"     => "Ï",  # &#0207; &#x00cf;
	"&ETH;"      => "Ð",  # &#0208; &#x00d0;
	"&Ntilde;"   => "Ñ",  # &#0209; &#x00d1;
	"&Ograve;"   => "Ò",  # &#0210; &#x00d2;
	"&Oacute;"   => "Ó",  # &#0211; &#x00d3;
	"&Ocirc;"    => "Ô",  # &#0212; &#x00d4;
	"&Otilde;"   => "Õ",  # &#0213; &#x00d5;
	"&Ouml;"     => "Ö",  # &#0214; &#x00d6;
	"&times;"    => "×",  # &#0215; &#x00d7;
	"&Oslash;"   => "Ø",  # &#0216; &#x00d8;
	"&Ugrave;"   => "Ù",  # &#0217; &#x00d9;
	"&Uacute;"   => "Ú",  # &#0218; &#x00da;
	"&Ucirc;"    => "Û",  # &#0219; &#x00db;
	"&Uuml;"     => "Ü",  # &#0220; &#x00dc;
	"&Yacute;"   => "Ý",  # &#0221; &#x00dd;
	"&THORN;"    => "Þ",  # &#0222; &#x00de;
	"&szlig;"    => "ß",  # &#0223; &#x00df;
	"&agrave;"   => "à",  # &#0224; &#x00e0;
	"&aacute;"   => "á",  # &#0225; &#x00e1;
	"&acirc;"    => "â",  # &#0226; &#x00e2;
	"&atilde;"   => "ã",  # &#0227; &#x00e3;
	"&auml;"     => "ä",  # &#0228; &#x00e4;
	"&aring;"    => "å",  # &#0229; &#x00e5;
	"&aelig;"    => "æ",  # &#0230; &#x00e6;
	"&ccedil;"   => "ç",  # &#0231; &#x00e7;
	"&egrave;"   => "è",  # &#0232; &#x00e8;
	"&eacute;"   => "é",  # &#0233; &#x00e9;
	"&ecirc;"    => "ê",  # &#0234; &#x00ea;
	"&euml;"     => "ë",  # &#0235; &#x00eb;
	"&igrave;"   => "ì",  # &#0236; &#x00ec;
	"&iacute;"   => "í",  # &#0237; &#x00ed;
	"&icirc;"    => "î",  # &#0238; &#x00ee;
	"&iuml;"     => "ï",  # &#0239; &#x00ef;
	"&eth;"      => "ð",  # &#0240; &#x00f0;
	"&ntilde;"   => "ñ",  # &#0241; &#x00f1;
	"&ograve;"   => "ò",  # &#0242; &#x00f2;
	"&oacute;"   => "ó",  # &#0243; &#x00f3;
	"&ocirc;"    => "ô",  # &#0244; &#x00f4;
	"&otilde;"   => "õ",  # &#0245; &#x00f5;
	"&ouml;"     => "ö",  # &#0246; &#x00f6;
	"&divide;"   => "÷",  # &#0247; &#x00f7;
	"&oslash;"   => "ø",  # &#0248; &#x00f8;
	"&ugrave;"   => "ù",  # &#0249; &#x00f9;
	"&uacute;"   => "ú",  # &#0250; &#x00fa;
	"&ucirc;"    => "û",  # &#0251; &#x00fb;
	"&uuml;"     => "ü",  # &#0252; &#x00fc;
	"&yacute;"   => "ý",  # &#0253; &#x00fd;
	"&thorn;"    => "þ",  # &#0254; &#x00fe;
	"&yuml;"     => "ÿ",  # &#0255; &#x00ff;
	"&OElig;"    => "Œ",  # &#0338; &#x0152;
	"&oelig;"    => "œ",  # &#0339; &#x0153;
	"&Scaron;"   => "Š",  # &#0352; &#x0160;
	"&scaron;"   => "š",  # &#0353; &#x0161;
	"&Yuml;"     => "Ÿ",  # &#0376; &#x0178;
	"&fnof;"     => "ƒ",  # &#0402; &#x0192;
	"&circ;"     => "ˆ",  # &#0710; &#x02c6;
	"&tilde;"    => "˜",  # &#0732; &#x02dc;
	"&Alpha;"    => "Α",  # &#0913; &#x0391;
	"&Beta;"     => "Β",  # &#0914; &#x0392;
	"&Gamma;"    => "Γ",  # &#0915; &#x0393;
	"&Delta;"    => "Δ",  # &#0916; &#x0394;
	"&Epsilon;"  => "Ε",  # &#0917; &#x0395;
	"&Zeta;"     => "Ζ",  # &#0918; &#x0396;
	"&Eta;"      => "Η",  # &#0919; &#x0397;
	"&Theta;"    => "Θ",  # &#0920; &#x0398;
	"&Iota;"     => "Ι",  # &#0921; &#x0399;
	"&Kappa;"    => "Κ",  # &#0922; &#x039a;
	"&Lambda;"   => "Λ",  # &#0923; &#x039b;
	"&Mu;"       => "Μ",  # &#0924; &#x039c;
	"&Nu;"       => "Ν",  # &#0925; &#x039d;
	"&Xi;"       => "Ξ",  # &#0926; &#x039e;
	"&Omicron;"  => "Ο",  # &#0927; &#x039f;
	"&Pi;"       => "Π",  # &#0928; &#x03a0;
	"&Rho;"      => "Ρ",  # &#0929; &#x03a1;
	"&Sigma;"    => "Σ",  # &#0931; &#x03a3;
	"&Tau;"      => "Τ",  # &#0932; &#x03a4;
	"&Upsilon;"  => "Υ",  # &#0933; &#x03a5;
	"&Phi;"      => "Φ",  # &#0934; &#x03a6;
	"&Chi;"      => "Χ",  # &#0935; &#x03a7;
	"&Psi;"      => "Ψ",  # &#0936; &#x03a8;
	"&Omega;"    => "Ω",  # &#0937; &#x03a9;
	"&alpha;"    => "α",  # &#0945; &#x03b1;
	"&beta;"     => "β",  # &#0946; &#x03b2;
	"&gamma;"    => "γ",  # &#0947; &#x03b3;
	"&delta;"    => "δ",  # &#0948; &#x03b4;
	"&epsilon;"  => "ε",  # &#0949; &#x03b5;
	"&zeta;"     => "ζ",  # &#0950; &#x03b6;
	"&eta;"      => "η",  # &#0951; &#x03b7;
	"&theta;"    => "θ",  # &#0952; &#x03b8;
	"&iota;"     => "ι",  # &#0953; &#x03b9;
	"&kappa;"    => "κ",  # &#0954; &#x03ba;
	"&lambda;"   => "λ",  # &#0955; &#x03bb;
	"&mu;"       => "μ",  # &#0956; &#x03bc;
	"&nu;"       => "ν",  # &#0957; &#x03bd;
	"&xi;"       => "ξ",  # &#0958; &#x03be;
	"&omicron;"  => "ο",  # &#0959; &#x03bf;
	"&pi;"       => "π",  # &#0960; &#x03c0;
	"&rho;"      => "ρ",  # &#0961; &#x03c1;
	"&sigmaf;"   => "ς",  # &#0962; &#x03c2;
	"&sigma;"    => "σ",  # &#0963; &#x03c3;
	"&tau;"      => "τ",  # &#0964; &#x03c4;
	"&upsilon;"  => "υ",  # &#0965; &#x03c5;
	"&phi;"      => "φ",  # &#0966; &#x03c6;
	"&chi;"      => "χ",  # &#0967; &#x03c7;
	"&psi;"      => "ψ",  # &#0968; &#x03c8;
	"&omega;"    => "ω",  # &#0969; &#x03c9;
	"&thetasym;" => "ϑ",  # &#0977; &#x03d1;
	"&upsih;"    => "ϒ",  # &#0978; &#x03d2;
	"&piv;"      => "ϖ",  # &#0982; &#x03d6;
	"&ensp;"     => " ", # &#8194; &#x2002;
	"&emsp;"     => " ", # &#8195; &#x2003;
	"&thsp;"     => " ", # &#8201; &#x2009;
	"&zwnj;"     => "‌", # &#8204; &#x200c;
	"&zwj;"      => "‍", # &#8205; &#x200d;
	"&lrm;"      => "‎", # &#8206; &#x200e;
	"&rlm;"      => "‏", # &#8207; &#x200f;
	"&ndash;"    => "–", # &#8211; &#x2013;
	"&mdash;"    => "—", # &#8212; &#x2014;
	"&lsquo;"    => "‘", # &#8216; &#x2018;
	"&rsquo;"    => "’", # &#8217; &#x2019;
	"&sbquo;"    => "‚", # &#8218; &#x201a;
	"&ldquo;"    => "“", # &#8220; &#x201c;
	"&rdquo;"    => "”", # &#8221; &#x201d;
	"&bdquo;"    => "„", # &#8222; &#x201e;
	"&dagger;"   => "†", # &#8224; &#x2020;
	"&Dagger;"   => "‡", # &#8225; &#x2021;
	"&bull;"     => "•", # &#8226; &#x2022;
	"&hellip;"   => "…", # &#8230; &#x2026;
	"&permil;"   => "‰", # &#8240; &#x2030;
	"&prime;"    => "′", # &#8242; &#x2032;
	"&Prime;"    => "″", # &#8243; &#x2033;
	"&lsaquo;"   => "‹", # &#8249; &#x2039;
	"&rsaquo;"   => "›", # &#8250; &#x203a;
	"&oline;"    => "‾", # &#8254; &#x203e;
	"&frasl;"    => "⁄", # &#8260; &#x2044;
	"&euro;"     => "€", # &#8364; &#x20ac;
	"&image;"    => "ℑ", # &#8465; &#x2111;
	"&weierp;"   => "℘", # &#8472; &#x2118;
	"&real;"     => "ℜ", # &#8476; &#x211c;
	"&trade;"    => "™", # &#8482; &#x2122;
	"&alefsym;"  => "ℵ", # &#8501; &#x2135;
	"&larr;"     => "←", # &#8592; &#x2190;
	"&uarr;"     => "↑", # &#8593; &#x2191;
	"&rarr;"     => "→", # &#8594; &#x2192;
	"&darr;"     => "↓", # &#8595; &#x2193;
	"&harr;"     => "↔", # &#8596; &#x2194;
	"&crarr;"    => "↵", # &#8629; &#x21b5;
	"&lArr;"     => "⇐", # &#8656; &#x21d0;
	"&uArr;"     => "⇑", # &#8657; &#x21d1;
	"&rArr;"     => "⇒", # &#8658; &#x21d2;
	"&dArr;"     => "⇓", # &#8659; &#x21d3;
	"&hArr;"     => "⇔", # &#8660; &#x21d4;
	"&forall;"   => "∀", # &#8704; &#x2200;
	"&part;"     => "∂", # &#8706; &#x2202;
	"&exist;"    => "∃", # &#8707; &#x2203;
	"&empty;"    => "∅", # &#8709; &#x2205;
	"&nabla;"    => "∇", # &#8711; &#x2207;
	"&isin;"     => "∈", # &#8712; &#x2208;
	"&notin;"    => "∉", # &#8713; &#x2209;
	"&ni;"       => "∋", # &#8715; &#x220b;
	"&prod;"     => "∏", # &#8719; &#x220f;
	"&sum;"      => "∑", # &#8721; &#x2211;
	"&minus;"    => "−", # &#8722; &#x2212;
	"&lowast;"   => "∗", # &#8727; &#x2217;
	"&radic;"    => "√", # &#8730; &#x221a;
	"&prop;"     => "∝", # &#8733; &#x221d;
	"&infin;"    => "∞", # &#8734; &#x221e;
	"&ang;"      => "∠", # &#8736; &#x2220;
	"&and;"      => "∧", # &#8743; &#x2227;
	"&or;"       => "∨", # &#8744; &#x2228;
	"&cap;"      => "∩", # &#8745; &#x2229;
	"&cup;"      => "∪", # &#8746; &#x222a;
	"&int;"      => "∫", # &#8747; &#x222b;
	"&there4;"   => "∴", # &#8756; &#x2234;
	"&sim;"      => "∼", # &#8764; &#x223c;
	"&cong;"     => "≅", # &#8773; &#x2245;
	"&asymp;"    => "≈", # &#8776; &#x2248;
	"&ne;"       => "≠", # &#8800; &#x2260;
	"&equiv;"    => "≡", # &#8801; &#x2261;
	"&le;"       => "≤", # &#8804; &#x2264;
	"&ge;"       => "≥", # &#8805; &#x2265;
	"&sub;"      => "⊂", # &#8834; &#x2282;
	"&sup;"      => "⊃", # &#8835; &#x2283;
	"&nsub;"     => "⊄", # &#8836; &#x2284;
	"&sube;"     => "⊆", # &#8838; &#x2286;
	"&supe;"     => "⊇", # &#8839; &#x2287;
	"&oplus;"    => "⊕", # &#8853; &#x2295;
	"&otimes;"   => "⊗", # &#8855; &#x2297;
	"&perp;"     => "⊥", # &#8869; &#x22a5;
	"&sdot;"     => "⋅", # &#8901; &#x22c5;
	"&lceil;"    => "⌈", # &#8968; &#x2308;
	"&rceil;"    => "⌉", # &#8969; &#x2309;
	"&lfloor;"   => "⌊", # &#8970; &#x230a;
	"&rfloor;"   => "⌋", # &#8971; &#x230b;
	"&lang;"     => "〈", # &#9001; &#x2329;
	"&rang;"     => "〉", # &#9002; &#x232a;
	"&loz;"      => "◊", # &#9674; &#x25ca;
	"&spades;"   => "♠", # &#9824; &#x2660;
	"&clubs;"    => "♣", # &#9827; &#x2663;
	"&hearts;"   => "♥", # &#9829; &#x2665;
	"&diams;"    => "♦", # &#9830; &#x2666;
);

#**************************************************************************
sub ishtml {
	my $htm = shift;

	return ($htm && ($htm =~ /<(p|pre|dir|div|br|ul|li|ol)\s*[^>]*>/is)) ? 1 : 0;
}

#******************************************************************
sub encuri {
	my($s) = @_;

	return '' unless defined $s;

	$s =~ s/([^A-Za-z0-9\-_.!~*'()])/sprintf("%%%02X", ord($1))/ge;
	
	return $s;
}

#******************************************************************
sub decuri {
	my($s) = @_;

	return '' unless defined $s;
	
	$s =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;

	return $s;
}

#**********************************************************************
sub _packc4decentis {
	my ($num) = @_;

	if ($num < 256) {
		return dectext(enctext(pack("C",$num), 'Latin1'));
	} else {
		return dectext(pack("U",$num));
	}
}

#**************************************************************************
sub decentis {
	my ($txt) = @_;

	if (defined($txt) && ($txt ne '')) {
		$txt =~ s/(&[a-zA-Z][0-9a-zA-Z]+;)/exists $ENTIS{$1} ? $ENTIS{$1} : $1 /ge;
		$txt =~ s/&#([0-9]+);/_packc4decentis($1)/ge;
		$txt =~ s/&#x([0-9a-fA-F]+);/_packc4decentis(hex($1))/ge;
	}

	return $txt;
}

#**************************************************************************
sub encentis {
	my ($txt) = @_;

	if (defined($txt) && ($txt ne '')) {
		$txt =~ s/&/&amp;/g;
		$txt =~ s/'/&apos;/g;
		$txt =~ s/"/&quot;/g;
		$txt =~ s/</&lt;/g;
		$txt =~ s/>/&gt;/g;
	}

	return $txt;
}

#******************************************************************
return 1;
