package Lingua::Pere::Misc::MIME;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);
@EXPORT = ();
@EXPORT_OK = qw(
	mime2mime
	mime2descr
	mime2ext
	magic2mime
);

use POSIX;

my %DESC  = (
	'application/vnd.oasis.opendocument.text'                  => 'OpenOffice.org (Ver 2) OpenDocument Text Document',
	'application/vnd.oasis.opendocument.text-template'         => 'OpenOffice.org (Ver 2) OpenDocument Text Document Template',
	'application/vnd.oasis.opendocument.text-web'              => 'OpenOffice.org (Ver 2) OpenDocument HTML Text Template',
	'application/vnd.oasis.opendocument.text-master'           => 'OpenOffice.org (Ver 2) OpenDocument Global Text Document',
	'application/vnd.oasis.opendocument.graphics'              => 'OpenOffice.org (Ver 2) OpenDocument Graphics Document',
	'application/vnd.oasis.opendocument.graphics-template'     => 'OpenOffice.org (Ver 2) OpenDocument Graphics Document Template',
	'application/vnd.oasis.opendocument.presentation'          => 'OpenOffice.org (Ver 2) OpenDocument Presentation',
	'application/vnd.oasis.opendocument.presentation-template' => 'OpenOffice.org (Ver 2) OpenDocument Presentation Template',
	'application/vnd.oasis.opendocument.spreadsheet'           => 'OpenOffice.org (Ver 2) OpenDocument Spreadsheet',
	'application/vnd.oasis.opendocument.spreadsheet-template'  => 'OpenOffice.org (Ver 2) OpenDocument Spreadsheet Template',
	'application/vnd.oasis.opendocument.chart'                 => 'OpenOffice.org (Ver 2) OpenDocument Chart',
	'application/vnd.oasis.opendocument.formula'               => 'OpenOffice.org (Ver 2) OpenDocument Formula',
	'application/vnd.oasis.opendocument.image'                 => 'OpenOffice.org (Ver 2) OpenDocument Image',
#	'application/vnd.oasis.opendocument.base'                  => 'OpenOffice.org (Ver 2) OpenDocument Database',
	'application/vnd.sun.xml.writer'                           => 'OpenOffice.org (Ver 1) Text Document',
	'application/vnd.sun.xml.writer.template'                  => 'OpenOffice.org (Ver 1) Text Document Template',
	'application/vnd.sun.xml.writer.global'                    => 'OpenOffice.org (Ver 1) Master Document',
	'application/vnd.sun.xml.calc'                             => 'OpenOffice.org (Ver 1) Spreadsheet',
	'application/vnd.sun.xml.calc.template'                    => 'OpenOffice.org (Ver 1) Spreadsheet Template',
	'application/vnd.sun.xml.impress'                          => 'OpenOffice.org (Ver 1) Presentation File',
	'application/vnd.sun.xml.impress.template'                 => 'OpenOffice.org (Ver 1) Presentation Template',
	'application/vnd.sun.xml.draw'                             => 'OpenOffice.org (Ver 1) Draw File',
	'application/vnd.sun.xml.draw.template'                    => 'OpenOffice.org (Ver 1) Drawing Template',
	'application/vnd.sun.xml.math'                             => 'OpenOffice.org (Ver 1) Math File',
	'application/vnd.sun.xml.base'                             => 'OpenOffice.org Database',
	'application/rtf'                                          => 'Rich Text Format File',
	'application/xhtml+xml'                                    => 'Extensible HyperText Markup Language File',
	'text/html'                                                => 'Hypertext Markup Language',
	'text/plain'                                               => 'Text File',
	'application/vnd.ms-word.xml'                              => 'Microsoft Word 2003 XML',
	'application/vnd.ms-excel.xml'                             => 'Microsoft Excel 2003 XML',
	'application/vnd.visio.xml'                                => 'Microsoft Visio 2003 XML',
	'application/vnd.docbook.xml'                              => 'DocBook Document',
	'application/vnd.pere.source'                              => 'Pere dictionary source file',
	'application/vnd.pere.dump'                                => 'Pere dictionary dump file',
);

my %EXTS  = (
	'application/vnd.oasis.opendocument.text'                  => [qw(odt)],
	'application/vnd.oasis.opendocument.text-template'         => [qw(ott)],
	'application/vnd.oasis.opendocument.text-web'              => [qw(oth)],
	'application/vnd.oasis.opendocument.text-master'           => [qw(odm)],
	'application/vnd.oasis.opendocument.graphics'              => [qw(odg)],
	'application/vnd.oasis.opendocument.graphics-template'     => [qw(otg)],
	'application/vnd.oasis.opendocument.presentation'          => [qw(odp)],
	'application/vnd.oasis.opendocument.presentation-template' => [qw(otp)],
	'application/vnd.oasis.opendocument.spreadsheet'           => [qw(ods)],
	'application/vnd.oasis.opendocument.spreadsheet-template'  => [qw(ots)],
	'application/vnd.oasis.opendocument.chart'                 => [qw(odc)],
	'application/vnd.oasis.opendocument.formula'               => [qw(odf)],
	'application/vnd.oasis.opendocument.image'                 => [qw(odi)],
#	'application/vnd.oasis.opendocument.base'                  => [qw()],
	'application/vnd.sun.xml.writer'                           => [qw(sxw)],
	'application/vnd.sun.xml.writer.template'                  => [qw(stw)],
	'application/vnd.sun.xml.writer.global'                    => [qw(sxg)],
	'application/vnd.sun.xml.calc'                             => [qw(sxc)],
	'application/vnd.sun.xml.calc.template'                    => [qw(stc)],
	'application/vnd.sun.xml.impress'                          => [qw(sxi)],
	'application/vnd.sun.xml.impress.template'                 => [qw(sti)],
	'application/vnd.sun.xml.draw'                             => [qw(sxd)],
	'application/vnd.sun.xml.draw.template'                    => [qw(std)],
	'application/vnd.sun.xml.math'                             => [qw(sxm)],
	'application/vnd.sun.xml.base'                             => [qw(odb)],
	'application/rtf'                                          => [qw(rtf)],
	'application/xhtml+xml'                                    => [qw(xhtml xht)],
	'text/html'                                                => [qw(html htm)],
	'text/plain'                                               => [qw(txt text asc)],
	'application/vnd.ms-word.xml'                              => [qw(xml)],
	'application/vnd.ms-excel.xml'                             => [qw(xml)],
	'application/vnd.visio.xml'                                => [qw(vdx)],
	'application/vnd.ms-word'                                  => [qw(doc)],
	'application/vnd.ms-excel'                                 => [qw(xls)],
	'application/vnd.ms-powerpoint'                            => [qw(ppt)],
	'application/vnd.docbook.xml'                              => [qw(xml)],
	'application/vnd.pere.source'                              => [qw(dmp dump)],
	'application/vnd.pere.dump'                                => [qw(src)],
);

my %NORM  = (
	'text/plain'                                                 => 'text/plain',
	'application/txt'                                            => 'text/plain',
	'browser/internal'                                           => 'text/plain',
	'text/anytext'                                               => 'text/plain',
	'widetext/plain'                                             => 'text/plain',
	'widetext/paragraph'                                         => 'text/plain',

	'text/html'                                                  => 'text/html',
	'application/xhtml+xml'                                      => 'application/xhtml+xml',

	'application/rtf'                                            => 'application/rtf',
	'application/x-rtf'                                          => 'application/rtf',
	'text/rtf'                                                   => 'application/rtf',
	'text/richtext'                                              => 'application/rtf',

	'application/vnd.oasis.opendocument.text'                    => 'application/vnd.oasis.opendocument.text',
	'application/vnd.oasis.opendocument.text-template'           => 'application/vnd.oasis.opendocument.text-template',
	'application/vnd.oasis.opendocument.text-web'                => 'application/vnd.oasis.opendocument.text-web',
	'application/vnd.oasis.opendocument.text-master'             => 'application/vnd.oasis.opendocument.text-master',
	'application/vnd.oasis.opendocument.graphics'                => 'application/vnd.oasis.opendocument.graphics',
	'application/vnd.oasis.opendocument.graphics-template'       => 'application/vnd.oasis.opendocument.graphics-template',
	'application/vnd.oasis.opendocument.presentation'            => 'application/vnd.oasis.opendocument.presentation',
	'application/vnd.oasis.opendocument.presentation-template'   => 'application/vnd.oasis.opendocument.presentation-template',
	'application/vnd.oasis.opendocument.spreadsheet'             => 'application/vnd.oasis.opendocument.spreadsheet',
	'application/vnd.oasis.opendocument.spreadsheet-template'    => 'application/vnd.oasis.opendocument.spreadsheet-template',
	'application/vnd.oasis.opendocument.chart'                   => 'application/vnd.oasis.opendocument.chart',
	'application/vnd.oasis.opendocument.formula'                 => 'application/vnd.oasis.opendocument.formula',
	'application/vnd.oasis.opendocument.image'                   => 'application/vnd.oasis.opendocument.image',
	'application/vnd.sun.xml.writer'                             => 'application/vnd.sun.xml.writer',
	'application/vnd.sun.xml.writer.template'                    => 'application/vnd.sun.xml.writer.template',
	'application/vnd.sun.xml.writer.global'                      => 'application/vnd.sun.xml.writer.global',
	'application/vnd.sun.xml.calc'                               => 'application/vnd.sun.xml.calc',
	'application/vnd.sun.xml.calc.template'                      => 'application/vnd.sun.xml.calc.template',
	'application/vnd.sun.xml.impress'                            => 'application/vnd.sun.xml.impress',
	'application/vnd.sun.xml.impress.template'                   => 'application/vnd.sun.xml.impress.template',
	'application/vnd.sun.xml.draw'                               => 'application/vnd.sun.xml.draw',
	'application/vnd.sun.xml.draw.template'                      => 'application/vnd.sun.xml.draw.template',
	'application/vnd.sun.xml.math'                               => 'application/vnd.sun.xml.math',
	'application/vnd.sun.xml.base'                               => 'application/vnd.sun.xml.base',

	'application/x-vnd.oasis.opendocument.text'                  => 'application/vnd.oasis.opendocument.text',
	'application/x-vnd.oasis.opendocument.text-template'         => 'application/vnd.oasis.opendocument.text-template',
	'application/x-vnd.oasis.opendocument.text-web'              => 'application/vnd.oasis.opendocument.text-web',
	'application/x-vnd.oasis.opendocument.text-master'           => 'application/vnd.oasis.opendocument.text-master',
	'application/x-vnd.oasis.opendocument.graphics'              => 'application/vnd.oasis.opendocument.graphics',
	'application/x-vnd.oasis.opendocument.graphics-template'     => 'application/vnd.oasis.opendocument.graphics-template',
	'application/x-vnd.oasis.opendocument.presentation'          => 'application/vnd.oasis.opendocument.presentation',
	'application/x-vnd.oasis.opendocument.presentation-template' => 'application/vnd.oasis.opendocument.presentation-template',
	'application/x-vnd.oasis.opendocument.spreadsheet'           => 'application/vnd.oasis.opendocument.spreadsheet',
	'application/x-vnd.oasis.opendocument.spreadsheet-template'  => 'application/vnd.oasis.opendocument.spreadsheet-template',
	'application/x-vnd.oasis.opendocument.chart'                 => 'application/vnd.oasis.opendocument.chart',
	'application/x-vnd.oasis.opendocument.formula'               => 'application/vnd.oasis.opendocument.formula',
	'application/x-vnd.oasis.opendocument.image'                 => 'application/vnd.oasis.opendocument.image',
	'application/x-vnd.sun.xml.writer'                           => 'application/vnd.sun.xml.writer',
	'application/x-vnd.sun.xml.writer.template'                  => 'application/vnd.sun.xml.writer.template',
	'application/x-vnd.sun.xml.writer.global'                    => 'application/vnd.sun.xml.writer.global',
	'application/x-vnd.sun.xml.calc'                             => 'application/vnd.sun.xml.calc',
	'application/x-vnd.sun.xml.calc.template'                    => 'application/vnd.sun.xml.calc.template',
	'application/x-vnd.sun.xml.impress'                          => 'application/vnd.sun.xml.impress',
	'application/x-vnd.sun.xml.impress.template'                 => 'application/vnd.sun.xml.impress.template',
	'application/x-vnd.sun.xml.draw'                             => 'application/vnd.sun.xml.draw',
	'application/x-vnd.sun.xml.draw.template'                    => 'application/vnd.sun.xml.draw.template',
	'application/x-vnd.sun.xml.math'                             => 'application/vnd.sun.xml.math',
	'application/x-vnd.sun.xml.base'                             => 'application/vnd.sun.xml.base',

	'application/vnd.ms-word'                                    => 'application/vnd.ms-word',
	'application/vnd.ms-excel'                                   => 'application/vnd.ms-excel',
	'application/vnd.ms-powerpoint'                              => 'application/vnd.ms-powerpoint',

	'application/vnd.ms-word.xml'                                => 'application/vnd.ms-word.xml',
	'application/vnd.ms-excel.xml'                               => 'application/vnd.ms-excel.xml',
	'application/vnd.docbook.xml'                                => 'application/vnd.docbook.xml',
	'application/vnd.pere.source'                                => 'application/vnd.pere.source',
	'application/vnd.pere.dump'                                  => 'application/vnd.pere.dump',
);

#******************************************************************
sub mime2mime {
	my ($mime) = @_;

	if (exists $NORM{$mime}) {
		return $NORM{$mime};
	} else {
		return $mime;
	}
}

#******************************************************************
sub mime2descr {
	my ($mime) = @_;

	my $mm = mime2mime($mime);
	if ($mm && (exists $DESC{$mm})) {
		return $DESC{$mm};
	} else {
		return '';
	}
}

#******************************************************************
sub mime2ext {
	my ($mime) = @_;
	
	my $mm = mime2mime($mime);
	if ($mm && exists($EXTS{$mime})) {
		return $EXTS{$mime};
	} else {
		return [];
	}
}

#******************************************************************
sub magic2mime {
	my ($data) = @_;

	if ($data =~ m/^\{\\rtf/) {
		return 'application/rtf';
	} elsif ($data =~ m/^\<\!DOCTYPE\ HTML/i) {
		return 'text/html';
	} elsif ($data =~ m/^\<HEAD/i) {
		return 'text/html';
	} elsif ($data =~ m/^\<TITLE/i) {
		return 'text/html';
	} elsif ($data =~ m/^\<HTML/i) {
		return 'text/html';
	} elsif ($data =~ m/^\<\!\-\-/) {
		return 'text/html';
	} elsif ($data =~ m/^\<H1/) {
		return 'text/html';
	} elsif ($data =~ m/^\/\*/) {
		return 'text/plain';
	} elsif ($data =~ m/^\/\//) {
		return 'text/plain';
	} elsif ($data =~ m/^\;\;/) {
		return 'text/plain';
	} elsif ($data =~ m/^\<\?xml/) {
		if ($data =~ m/<\?mso-application progid="Word\.Document"/) {
			return 'application/vnd.ms-word.xml';
		} elsif ($data =~ m/<\?mso-application progid="Excel\.Sheet"/) {
			return 'application/vnd.ms-excel.xml';
		} elsif ($data =~ m/<!DOCTYPE\s+[^>]+?(docbookx\.dtd|-\/\/OASIS\/\/DTD\s+DocBook)/s) {
			return 'application/vnd.docbook.xml';
		} elsif ($data =~ m/<[^>]+?xmlns="http:\/\/docbook\.org\/ns\/docbook"/s) {
			return 'application/vnd.docbook.xml';
		} else {
			return 'text/xml';
		}
	} elsif ($data =~ m/^PK\003\004/) {
		if ($data =~ m/^.{30}mimetype(.+?)PK/) {
			my $tp = $1;
			if (exists $NORM{$tp}) {
				return $NORM{$tp};
			} else {
				return 'application/x-zip';
			}
		} else {
			return 'application/x-zip';
		}
	} elsif ($data =~ m/^PERE_DICT_SOURCE/) {
		return 'application/vnd.pere.source';
	} elsif ($data =~ m/^PERE_DICT_DUMP/) {
		return 'application/vnd.pere.dump';
	}
	return '';
}

#**************************************************************************
return 1;
