package Lingua::Pere::Misc::Nums;

require 5.000;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT_OK);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);

@EXPORT_OK = qw(
	cleannumb
	cleanchar
	cleanshrt
	cleanlong
	gennum
	chknum
	chkint
	ceils
	min
	mina
	mins
	minsa
	max
	maxa
	maxs
	maxsa
	range
	lnlim
	dflim
	log2
	compnum
	randnum
);

use POSIX;

#**************************************************************************
sub cleannumb {
	my ($s, $d, $l, $r) = @_;

	if (defined($s) && ($s ne '')) {
		$s =~ s/[^[:digit:]]+//gs;
		return ($s ne '') 
			? (defined($l) && defined($r)) 
				? range($s, $l, $r) 
				: $s
			: $d;
	} else {
		return $d;
	}
}

#**************************************************************************
sub cleanchar {
	my ($s, $d) = @_;

	return cleannumb($s, $d, 0, UCHAR_MAX);
}

#**************************************************************************
sub cleanshrt {
	my ($s, $d) = @_;

	return cleannumb($s, $d, 0, SHRT_MAX);
}

#**************************************************************************
sub cleanlong {
	my ($s, $d) = @_;

	return cleannumb($s, $d, 0, LONG_MAX);
}

#**************************************************************************
sub gennum {
	my ($len) = @_;
	$len ||= 0;
	
	$len =~ s/[^0-9]//g;
	$len = 6 unless ($len);

	my $num = '';
	for (my $i = 1; $i <= $len; $i++) {
		$num .= int(rand(10));
	}

	return $num;
}

#**************************************************************************
sub chknum {
	my ($n, $d) = @_;
	
	if (defined($n) && ($n ne '')) {
		my ($num, $unp) = strtod($n);
	
		return (defined($num) && ($num ne '')) ? $num : $d;
	} else {
		return $d;
	}
}

#**************************************************************************
sub chkint {
	my ($n, $d) = @_;

	if (defined($n) && ($n ne '')) {
		my ($num, $unp) = strtod($n);
	
		$n = (defined($num) && ($num ne '')) ? $num : $d;
	} else {
		$n = $d;
	}

	return ceil($n);
}

#**************************************************************************
sub ceils {
	my ($x, $y) = @_;

	return ($x % $y == 0) ? ($x/$y) : int($x/$y+1);
}

#**************************************************************************
sub min {
	my ($x, $y) = @_;

	return (defined($x))
		? (defined($y))
			? ($x > $y)
				? $y 
				: $x
			: $x
		: $y;
}

#**************************************************************************
sub mina {
	my $x = shift(@_);

	foreach my $y (@_) {
		$x = (defined($x))
			? (defined($y))
				? ($x > $y) 
					? $y 
					: $x
				: $x
			: $y;
	}
	
	return $x
}

#**************************************************************************
sub mins {
	my ($x, $y) = @_;

	return (defined($x))
		? (defined($y))
			? ($x gt $y)
				? $y 
				: $x
			: $x
		: $y;
}

#**************************************************************************
sub minsa {
	my $x = shift(@_);

	foreach my $y (@_) {
		$x = (defined($x))
			? (defined($y))
				? ($x gt $y) 
					? $y 
					: $x
				: $x
			: $y;
	}
	
	return $x
}

#**************************************************************************
sub max {
	my ($x, $y) = @_;

	return (defined($x))
		? (defined($y))
			? ($x < $y)
				? $y 
				: $x
			: $x
		: $y;
}

#**************************************************************************
sub maxa {
	my $x = shift(@_);

	foreach my $y (@_) {
		$x = (defined($x))
			? (defined($y))
				? ($x < $y) 
					? $y 
					: $x
				: $x
			: $y;
	}
	
	return $x
}

#**************************************************************************
sub maxs {
	my ($x, $y) = @_;

	return (defined($x))
		? (defined($y))
			? ($x lt $y)
				? $y 
				: $x
			: $x
		: $y;
}

#**************************************************************************
sub maxsa {
	my $x = shift(@_);

	foreach my $y (@_) {
		$x = (defined($x))
			? (defined($y))
				? ($x lt $y) 
					? $y 
					: $x
				: $x
			: $y;
	}
	
	return $x
}

#**************************************************************************
sub range {
	my ($x, $l, $r) = @_;

	return max(min($x, $r), $l);
}

#**************************************************************************
sub lnlim {
	my ($li, $lo, $ln) = @_;
	$ln ||= 13;

	return (($li > 0) && ($li <= $ln) && ($lo > 0) && ($lo <= $ln)) ? 1 : 0;
	
}

#**************************************************************************
sub dflim {
	my ($li, $lo, $df) = @_;
	$df ||= 3; #  1,2,3, 5, 7,11,13:      

	return ($df*min($li, $lo) >= max($li, $lo)) ? 1 : 0;
	
}

#**************************************************************************
#!!! log10 - defined in POSIX !!!
#**************************************************************************
sub log2 {
	my $n = shift;
	return log($n)/log(2);
}

#**************************************************************************
sub compnum {
	my ($num1, $num2) = @_;
	
	return $num1 <=> $num2;
}

#**************************************************************************
sub randnum {
	my ($len) = @_;
	$len ||= 0;
	
	$len =~ s/[^0-9]//g;
	$len = 6 unless ($len);

	my $num = '';
	for (my $i = 1; $i <= $len; $i++) {
		$num .= int(rand(10));
	}

	return $num;
}

#**************************************************************************
return 1;
