package Lingua::Pere::Misc::Text;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);
@EXPORT = ();
@EXPORT_OK = qw(
	enctext
	dectext
	codtext
	cleantext
	cleanstr
	preptext
	text2parse
	parse2text
	parse2words
	readtext
	writetext
);

use POSIX;
use Encode qw(
	encode
	decode
	encode_utf8
	decode_utf8
	from_to
	is_utf8
);
use Lingua::Pere::Misc::File qw(
	readfile
	writefile
);

my $BLANK = "[:blank:][:space:][:cntrl:]";
my $SEPAR = "\x{001f}";

#**************************************************************************
sub enctext {
	my ($txt, $enc) = @_;
	
	unless (is_utf8($txt)) {
		if ($enc) {
			$txt = decode($enc, $txt);
		} else {
			$txt = decode_utf8($txt);
		}
	}
	
	return $txt;
}

#**************************************************************************
sub dectext {
	my ($txt, $enc) = @_;
	
	if (is_utf8($txt)) {
		if ($enc) {
			$txt = encode($enc, $txt);
		} else {
			$txt = encode_utf8($txt);
		}
	}
	
	return $txt;
}

#**************************************************************************
sub codtext {
	my ($txt, $enc) = @_;
	
	if ($enc) {
		my $len = from_to($txt, $enc, 'UTF-8');
		unless (defined $len) {
			$txt = undef;
		}
	}
	
	return $txt;
}

#**************************************************************************
sub preptext {
	my ($txt) = @_;

	my $ltx = '';	
	if ($txt =~ m/^([$BLANK]+)(.+?)$/s) {
		$ltx = $1;
		$txt = $2;
	}

	my $rtx = '';	
	if ($txt =~ m/^(.+?)([$BLANK]+)$/s) {
		$txt = $1;
		$rtx = $2;
	}

	return ($ltx, $txt, $rtx);
}

#**************************************************************************
sub cleantext {
	my ($txt) = @_;
	
	$txt =~ s/^[$BLANK]+//s;
	$txt =~ s/[$BLANK]+$//s;
	$txt =~ s/[$BLANK]+/ /gs;
	
	return $txt;
}

#**************************************************************************
sub cleanstr {
	my ($s, $l, $d) = @_;
	if (defined($s) && ($s ne '')) {
		$s =~ s/[$BLANK]+/ /gs;
		$s =~ s/^[$BLANK]+//s;
		$s =~ s/[$BLANK]+$//s;
		return ($l) ? substr($s, 0, $l) : $s;
	} else {
		return $d;
	}
}

#**************************************************************************
#=cmt
sub text2parse {
	my ($txt, $res) = @_;
	$res ||= [];

	my @wa = split(m/[$BLANK]+/s,$txt);
	foreach my $s (@wa) {
		next if ($s eq '');

		if ($s =~ m/^([^[:alnum:]]+)(.+?)$/) {
			push(@{$res}, $1.$SEPAR);
			$s = $2;
		}
		if ($s =~ m/^(.+?)([^[:alnum:]]+)$/) {
			push(@{$res}, $1);
			$s = $SEPAR.$2;
		}
		push(@{$res}, $s);
	};

	return $res;
}
#=cut
#**************************************************************************
=cmt
sub text2parse {
	my ($txt, $res) = @_;
	$res ||= [];

	my @wa = split(m/[$BLANK]+/s,$txt);
	my $d;
	foreach my $s (@wa) {
		next if ($s eq '');

		if ($s =~ m/^([^[:alnum:]]+)(.+?)$/) {
			$d = $1.$SEPAR;
			$s = $2;
			push(@{$res}, $d);
		}
		if ($s =~ m/^(.+?)([^[:alnum:]]+)$/) {
			$s = $1;
			$d = $2;
		} else {
			$d = '';
		}

		my @a = split(m/([^\'[:alnum:]]+)/, $s);
		foreach my $w (@a) {
			next if ($w eq '');
			if ($w =~ m/[[:alnum:]]/) {
				push(@{$res}, $w);
			} else {
				push(@{$res}, $SEPAR.$w.$SEPAR);
			}
		}

		push(@{$res}, $SEPAR.$d) if ($d ne '');
	};

	return $res;
}
=cut
#**************************************************************************
=cmt
sub text2parse {
	my ($txt, $res) = @_;
	$res ||= [];

	my @wa = split(m/([-'[:alnum:]]+)/,$txt);
	my $d;
	foreach my $s (@wa) {
		next if ($s eq '');

		if ($s =~ m/[[:alnum:]]/) {
			if ($s =~ /^([^[:alnum:]]+)(.+?)$/) {
				$d = $1;
				$s = $2;
				if ((scalar(@{$res}) < 1) || ($res->[$#{$res}] =~ /[[:alnum:]]/)) {
					push(@{$res}, $d);
				} else {
					$res->[$#{$res}] .= $d;
				}
			}

			if ($s =~ /^(.+?)([^[:alnum:]]+)$/) {
				push(@{$res}, $1, $2);
			} else {
				push(@{$res}, $s);
			}
		} elsif ((scalar(@{$res}) < 1) || ($res->[$#{$res}] =~ /[[:alnum:]]/)) {
			push(@{$res}, $s);
		} else {
			$res->[$#{$res}] .= $s;
		}
	};

	return $res;
}
=cut
#**************************************************************************
#=cmt
sub parse2text {
	my ($prs, $txt, $min, $max) = @_;
	
	$min ||= 0;
	$max ||= $#{$prs};
	$txt ||= '';

	my $s = join(' ', @{$prs}[$min..$max]);
	
	$s =~ s/\s+[$SEPAR]+//g;
	$s =~ s/[$SEPAR]+\s+//g;
	$s =~ s/[$SEPAR]+//g;

	$txt .= $s;

	return $txt;
}
#=cut
#**************************************************************************
=cmt
sub parse2text {
	my ($prs, $txt, $min, $max) = @_;
	
	$min ||= 0;
	$max ||= $#{$prs};
	$txt = '' unless(defined($txt));

	$txt .= join('', @{$prs}[$min..$max]);

	return $txt;
}
=cut
#**************************************************************************
sub parse2words {
	my ($prs, $wds) = @_;
	$wds ||= [];

	foreach my $wd (@{$prs}) {
		if (($wd ne '') && ($wd =~ /[[:alnum:]]/)) {
			push(@{$wds}, $wd);
		}
	}
	
	return $wds;
}

#**************************************************************************
sub readtext {
	my ($fil, $enc) = @_;

	my $txt = readfile($fil);
	
	return (defined($txt) && ($txt ne '')) ? text2parse(cleantext(enctext($txt, $enc))) : undef;
}

#**************************************************************************
sub writetext {
	my ($fil, $prs, $enc) = @_;
	
	writefile($fil, dectext(parse2text($prs), $enc));
}

#**************************************************************************
return 1;
__END__
