package Lingua::Pere::Util::RTF;

require 5.000;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);
@EXPORT = ();
@EXPORT_OK = qw(
);
#	isrtf

use POSIX;
use RTF::Tokenizer;
use Lingua::Pere::Misc::Text qw(
	enctext
	dectext
	preptext
	text2parse
	parse2text
);

my %LANGS = (
	 1078 => "cp1252", # Afrikaans
	 1052 => "cp1250", # Albanian
	 5121 => "cp1256", # Arabic (Algeria)
	15361 => "cp1256", # Arabic (Bahrain)
	 3073 => "cp1256", # Arabic (Egypt)
	 2049 => "cp1256", # Arabic (Iraq)
	11265 => "cp1256", # Arabic (Jordan)
	13313 => "cp1256", # Arabic (Kuwait)
	12289 => "cp1256", # Arabic (Lebanon)
	 4097 => "cp1256", # Arabic (Libya)
	 6145 => "cp1256", # Arabic (Morocco)
	 8193 => "cp1256", # Arabic (Oman)
	16385 => "cp1256", # Arabic (Qatar)
	 1025 => "cp1256", # Arabic (Saudi Arabia)
	10241 => "cp1256", # Arabic (Syria)
	 7169 => "cp1256", # Arabic (Tunisia)
	14337 => "cp1256", # Arabic (U.A.E.)
	 9217 => "cp1256", # Arabic (Yemen)
	 1067 => "cp1252", # Armenian
	 2092 => "cp1251", # Azeri (Cyrillic)
	 1068 => "cp1254", # Azeri (Latin)
	 1069 => "cp1252", # Basque
	 1059 => "cp1251", # Belarusian
	 1093 => "cp1252", # Bengali (India)
	 5146 => "cp1250", # Bosnian (Latin, Bosnia and Herzegovina)
	 1026 => "cp1251", # Bulgarian
	 1027 => "cp1252", # Catalan
	 3076 =>  "cp950", # Chinese (Hong Kong S.A.R.)
	 5124 =>  "cp950", # Chinese (Macau S.A.R.)
	 2052 =>  "cp936", # Chinese (PRC)
	 4100 =>  "cp936", # Chinese (Singapore)
	 1028 =>  "cp950", # Chinese (Taiwan)
	 1050 => "cp1250", # Croatian
	 4122 => "cp1250", # Croatian (Bosnia and Herzegovina)
	 1029 => "cp1250", # Czech
	 1030 => "cp1252", # Danish
	 1125 => "cp1252", # Divehi
	 2067 => "cp1252", # Dutch (Belgium)
	 1043 => "cp1252", # Dutch (Netherlands)
	 3081 => "cp1252", # English (Australia)
	10249 => "cp1252", # English (Belize)
	 4105 => "cp1252", # English (Canada)
	 9225 => "cp1252", # English (Caribbean)
	 6153 => "cp1252", # English (Ireland)
	 8201 => "cp1252", # English (Jamaica)
	 5129 => "cp1252", # English (New Zealand)
	13321 => "cp1252", # English (Philippines)
	 7177 => "cp1252", # English (South Africa)
	11273 => "cp1252", # English (Trinidad)
	 2057 => "cp1252", # English (United Kingdom)
	 1033 => "cp1252", # English (United States)
	12297 => "cp1252", # English (Zimbabwe)
	 1061 => "cp1257", # Estonian
	 1080 => "cp1252", # Faeroese
	 1065 => "cp1256", # Farsi
	 1035 => "cp1252", # Finnish
	 2060 => "cp1252", # French (Belgium)
	 3084 => "cp1252", # French (Canada)
	 1036 => "cp1252", # French (France)
	 5132 => "cp1252", # French (Luxembourg)
	 6156 => "cp1252", # French (Monaco)
	 4108 => "cp1252", # French (Switzerland)
	 1071 => "cp1251", # FYRO Macedonian
	 1110 => "cp1252", # Galician
	 1079 => "cp1252", # Georgian
	 3079 => "cp1252", # German (Austria)
	 1031 => "cp1252", # German (Germany)
	 5127 => "cp1252", # German (Liechtenstein)
	 4103 => "cp1252", # German (Luxembourg)
	 2055 => "cp1252", # German (Switzerland)
	 1032 => "cp1253", # Greek
	 1095 => "cp1252", # Gujarati
	 1037 => "cp1255", # Hebrew
	 1081 => "cp1252", # Hindi
	 1038 => "cp1250", # Hungarian
	 1039 => "cp1252", # Icelandic
	 1057 => "cp1252", # Indonesian
	 1040 => "cp1252", # Italian (Italy)
	 2064 => "cp1252", # Italian (Switzerland)
	 1041 =>  "cp932", # Japanese
	 1099 => "cp1252", # Kannada
	 1087 => "cp1251", # Kazakh
	 1111 => "cp1252", # Konkani
	 1042 =>  "cp949", # Korean
	 1088 => "cp1251", # Kyrgyz (Cyrillic)
	 1062 => "cp1257", # Latvian
	 1063 => "cp1257", # Lithuanian
	 2110 => "cp1252", # Malay (Brunei Darussalam)
	 1086 => "cp1252", # Malay (Malaysia)
	 1100 => "cp1252", # Malayalam (India)
	 1082 => "cp1252", # Maltese
	 1153 => "cp1252", # Maori
	 1102 => "cp1252", # Marathi
	 1104 => "cp1251", # Mongolian (Cyrillic)
	 1132 => "cp1252", # Northern Sotho
	 1044 => "cp1252", # Norwegian (Bokmal)
	 2068 => "cp1252", # Norwegian (Nynorsk)
	 1045 => "cp1250", # Polish
	 1046 => "cp1252", # Portuguese (Brazil)
	 2070 => "cp1252", # Portuguese (Portugal)
	 1094 => "cp1252", # Punjabi
	 1131 => "cp1252", # Quechua (Bolivia)
	 2155 => "cp1252", # Quechua (Ecuador)
	 3179 => "cp1252", # Quechua (Peru)
	 1048 => "cp1250", # Romanian
	 1049 => "cp1251", # Russian
	 9275 => "cp1252", # Sami, Inari (Finland)
	 4155 => "cp1252", # Sami, Lule (Norway)
	 5179 => "cp1252", # Sami, Lule (Sweden)
	 3131 => "cp1252", # Sami, Northern (Finland)
	 1083 => "cp1252", # Sami, Northern (Norway)
	 2107 => "cp1252", # Sami, Northern (Sweden)
	 8251 => "cp1252", # Sami, Skolt (Finland)
	 6203 => "cp1252", # Sami, Southern (Norway)
	 7227 => "cp1252", # Sami, Southern (Sweden)
	 1103 => "cp1252", # Sanskrit
	 3098 => "cp1251", # Serbian (Cyrillic)
	 7194 => "cp1251", # Serbian (Cyrillic, Bosnia and Herzegovina)
	 2074 => "cp1250", # Serbian (Latin)
	 6170 => "cp1250", # Serbian (Latin, Bosnia and Herzegovina)
	 1051 => "cp1250", # Slovak
	 1060 => "cp1250", # Slovenian
	11274 => "cp1252", # Spanish (Argentina)
	16394 => "cp1252", # Spanish (Bolivia)
	13322 => "cp1252", # Spanish (Chile)
	 9226 => "cp1252", # Spanish (Colombia)
	 5130 => "cp1252", # Spanish (Costa Rica)
	 7178 => "cp1252", # Spanish (Dominican Republic)
	12298 => "cp1252", # Spanish (Ecuador)
	17418 => "cp1252", # Spanish (El Salvador)
	 4106 => "cp1252", # Spanish (Guatemala)
	18442 => "cp1252", # Spanish (Honduras)
	 3082 => "cp1252", # Spanish (International Sort)
	 2058 => "cp1252", # Spanish (Mexico)
	19466 => "cp1252", # Spanish (Nicaragua)
	 6154 => "cp1252", # Spanish (Panama)
	15370 => "cp1252", # Spanish (Paraguay)
	10250 => "cp1252", # Spanish (Peru)
	20490 => "cp1252", # Spanish (Puerto Rico)
	 1034 => "cp1252", # Spanish (Traditional Sort)
	14346 => "cp1252", # Spanish (Uruguay)
	 8202 => "cp1252", # Spanish (Venezuela)
	 1089 => "cp1252", # Swahili
	 1053 => "cp1252", # Swedish
	 2077 => "cp1252", # Swedish (Finland)
	 1114 => "cp1252", # Syriac
	 1097 => "cp1252", # Tamil
	 1092 => "cp1251", # Tatar
	 1098 => "cp1252", # Telugu
	 1054 =>  "cp874", # Thai
	 1074 => "cp1252", # Tswana
	 1055 => "cp1254", # Turkish
	 1058 => "cp1251", # Ukrainian
	 1056 => "cp1256", # Urdu
	 2115 => "cp1251", # Uzbek (Cyrillic)
	 1091 => "cp1254", # Uzbek (Latin)
	 1066 => "cp1258", # Vietnamese
	 1106 => "cp1252", # Welsh
	 1076 => "cp1252", # Xhosa
	 1077 => "cp1252", # Zulu
);

my %NOTEXT = (
	'*'            => 1,
	'leveltext'    => 1,
	'listtable'    => 1,
	'istpicture'   => 1,
	'levelnumbers' => 1,
	'bkmkstart'    => 1,
	'bkmkend'      => 1,
	'styleseet'    => 1,
	'colortbl'     => 1,

	'pict'         => 1,
	'bin'          => 1,
	'object'       => 1,
	'do'           => 1,
	'shp'          => 1,
	'spp'          => 1,
);

my %CMNDS = (
	emdash    => '-', # --
	endash    => '-',
	emspace   => ' ',
	enspace   => ' ',
	qmspace   => ' ',
	lquote    => "'",
	rquote    => "'",
	ldblquote => '"',
	rdblquote => '"',
	"~"       => ' ',
	"_"       => '',
	"-"       => '',
);

#******************************************************************
sub enc {
	my ($self) = @_;
	
	return $self->{enca}->[$#{$self->{enca}}];
}

#******************************************************************
sub setenc {
	my ($self, $enc) = @_;
	
	if (defined($enc) && ($enc ne '')) {
		$self->{enca}->[$#{$self->{enca}}] = $enc;
	}
}

#******************************************************************
sub istxt {
	my ($self, $txt) = @_;
	
	my $old = $self->{istext}->[$#{$self->{istext}}];
	if (defined($txt)) {
		$self->{istext}->[$#{$self->{istext}}] = ($txt) ? 1 : 0;
	}
	
	return $old;
}

#******************************************************************
sub isuni {
	my ($self) = @_;
	
	return $self->{unicode}--;
}

#******************************************************************
sub add2row { 
	my ($self, $res) = @_;

	if (defined($res) && ($res ne "")) {
		$self->{row} .= $res;
	}
}

#******************************************************************
sub flushrow { 
	my ($self) = @_;

	if ($self->{row} ne '') {
		my $enc = $self->enc;
		my @arw = split('', &{$self->{func}}($self->{row}));
		foreach my $cha (@arw) {
			my $num = ord($cha);
			if ($num < 256) {
				unless ($cha =~ m/[- _.,\/@#$%!^&*()+=~`|?[:alnum:]]/) {
					$cha = sprintf("\\\'%02x", $num);
				}
			} else {
				my $sim = dectext($cha, $enc);
				$sim = '?' unless (defined $sim);
				$cha = sprintf("\\u%d\\\'%02x", $num, ord($sim));
			}
		}
		$self->{buf} .= ' '.join('', @arw);

		$self->{row} = '';
	}
}

#******************************************************************
sub buf {
	my ($self) = @_;
	
	return $self->{buf};
}

#******************************************************************
sub add2buf { 
	my ($self, $res) = @_;

	if (defined($res) && ($res ne "")) {
		$self->flushrow;
		$self->{buf} .= $res;
	}
}

#******************************************************************
sub execute { 
	my ($self, $org) = @_;

#	my $prs = RTF::Tokenizer->new(string => $org, sloppy => 1);
	my $prs = RTF::Tokenizer->new(string => $org);
	while (1) {
		my ($tok, $typ, $arg) = $prs->get_token();
		&{$self->{progres}};

		if ($tok eq 'control') {
			my $ist = $self->istxt;
			if ($ist && exists($CMNDS{$typ})) {
				$self->add2row(enctext($CMNDS{$typ}));
				next;
			} elsif ($ist && ($typ eq 'u')) {
				$self->add2row(enctext(chr($arg)));
				$self->{unicode} = $self->{skipchr};
				next;
			} elsif ($ist && ($typ eq "'")) {
				if ($self->isuni < 1) {
					$self->add2row(enctext(chr(hex($arg)), $self->enc));
				}
				next;
			} elsif (exists($NOTEXT{$typ})) {
					$self->istxt(0);
			} elsif ($typ eq 'ansicpg') {
				$self->{ansicpg} = "cp$arg";
			} elsif ($typ eq 'deflang') {
				if (exists($LANGS{$arg})) {
					my $lng = $LANGS{$arg};
					$self->{deflang} = $lng;
					if ($self->{ansicpg} eq '') {
						$self->setenc($lng);
					}
				}
			} elsif ($typ eq 'lang') {
				if (($self->{ansicpg} eq '') && exists($LANGS{$arg})) {
					$self->setenc($LANGS{$arg});
				}
			} elsif ($typ eq 'plain') {
				if ($self->{ansicpg} ne '') {
					$self->setenc($self->{ansicpg});
				} elsif ($self->{deflang} ne '') {
					$self->setenc($self->{deflang});
				} else {
					$self->setenc($self->{defenc});
				}
			} elsif ($typ eq 'pard') {
				$self->istxt(1);
			} elsif ($typ eq 'uc') {
				$self->{skipchr} = $arg;
			}
			$self->add2buf("\\$typ$arg");
		} elsif ($tok eq 'group') {
			if ($typ) {
				$self->add2buf('{');
				push(@{$self->{enca}}, $self->{enca}->[$#{$self->{enca}}]);
				push(@{$self->{istext}}, $self->{istext}->[$#{$self->{istext}}]);
			} else {
				pop(@{$self->{enca}}) if (scalar(@{$self->{enca}}) > 1);
				pop(@{$self->{istext}}) if (scalar(@{$self->{istext}}) > 1);
				$self->add2buf('}');
			}
		} elsif ($tok eq 'text') {
			if ($self->istxt) {
				$self->add2row(enctext($typ, $self->enc));
			} else {
				$self->add2buf(' '.$typ);
			}
		} elsif ($tok eq 'eof') {
			last;
		} else {
			last;
		}
	}

	return $self->buf;
}

#******************************************************************
sub new { 
	my ($class, $prg, $fun, $enc) = @_;

	my $self = bless {
		defenc  => (defined($enc) && ($enc ne '')) ? $enc : "cp1252",
		istext  => [0],
		row     => '',
		buf     => '',
		skipchr => 1,
		unicode => 0,
		ansicpg => '',
		deflang => '',
		progres => $prg || sub {},
		func    => $fun || sub { return $_[0] },
	}, $class;

	$self->{enca} = [$self->{defenc}];

	return $self;
}

#******************************************************************
return 1;
__END__
