package Lingua::Pere::Util::XML;

require 5.000;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);

use Exporter;
use XML::Parser::Expat;

$VERSION = "0.2.0";
@ISA = qw(Exporter XML::Parser::Expat);
@EXPORT = ();
@EXPORT_OK = qw(
);
#	isrtf

use POSIX;
use XML::Parser::Expat;
use Lingua::Pere::Dict::Trans;
use Lingua::Pere::Misc::Text qw(
	enctext
	dectext
	preptext
	text2parse
	parse2text
);
use Lingua::Pere::Misc::File qw(
	openfile
);
use Lingua::Pere::Misc::HTML qw(
	encentis
	decentis
);

#**************************************************************************
sub dic {
	my ($self) = @_;
	
	return $self->{PERE}->{dic};
}

#**************************************************************************
sub prg {
	my ($self) = @_;
	
	return $self->{PERE}->{prg};
}

#**************************************************************************
sub cas {
	my ($self) = @_;
	
	return $self->{PERE}->{cas};
}

#**************************************************************************
sub ihn {
	my ($self) = @_;
	
	return $self->{PERE}->{ihn};
}

#**************************************************************************
sub ohn {
	my ($self) = @_;
	
	return $self->{PERE}->{ohn};
}

#**************************************************************************
sub enc {
	my ($self) = @_;
	
	return $self->{PERE}->{enc};
}

#**************************************************************************
sub oka {
	my ($self) = @_;
	
	return $self->{PERE}->{oka};
}

#**************************************************************************
sub huk {
	my ($self) = @_;
	
	my $oka = $self->{PERE}->{huk}->(@_);
	$self->{PERE}->{oka} += $oka;
}

#**************************************************************************
sub print {
	my $self = shift;
	
	return $self->{PERE}->{ohn}->print(@_);
}

#**************************************************************************
sub printf {
	my $self = shift;
	
	return $self->{PERE}->{ohn}->printf(@_);
}

#**************************************************************************
sub buf {
	my ($self) = @_;
	
	return $self->{PERE}->{buf};
}

#**************************************************************************
sub add2buf {
	my ($self, $val) = @_;
	
	$self->{PERE}->{buf} .= $val;
}

#**************************************************************************
sub flushbuf {
	my ($self) = @_;
	
	my $row = $self->{PERE}->{buf};
	if ($row ne '') {
		$self->{PERE}->{buf} = '';
#		if (defined($row) && ($row ne '')) {
		if (defined($row) && ($row ne '') && ($row =~ m/[[:alnum:]]/)) {
			my ($ltx, $txt, $rtx) = preptext($row);
			my $tra = [];
			foreach my $rwt (@{$self->dic->translate(text2parse($txt), $self->cas)}) {
#---------------------------------------------
#				if ($rwt->[0] < 1) {
#					foreach my $rw (@{$rwt->[1]}) {
#						if ($rw =~ m/[[:alpha:]]/) {
#							$rw = $STXU.$rw.$ETXU;
#						}
#					}
#				}
#---------------------------------------------
				push(@{$tra}, @{$rwt->[1]});
			}
			$row = encentis(dectext($ltx.parse2text($tra).$rtx));
#---------------------------------------------
#			$row =~ s/[$STXA]+/$SOOO/g;
#			$row =~ s/[$ETXA]+/$EOOO/g;
#---------------------------------------------
			$self->print($row);
		} else {
			$self->print(encentis(dectext($row)));
		}
		&{$self->prg};
	}
}

#**************************************************************************
sub hStart {
	my ($self, $elm, %att) = @_;

	my $atr = '';
	while (my ($key, $val) = each %att) {
		$atr .= sprintf(" %s=\"%s\"", dectext($key), encentis(dectext($val)));
	}
	$self->flushbuf;
	$self->huk(1, $elm, \%att);
	$self->printf("<%s%s>", $elm, $atr);
}

#**************************************************************************
sub hEnd {
	my ($self, $elm) = @_;

	$self->flushbuf;
	$self->huk(0, $elm);
	$self->printf("</%s>", $elm);
}

#**************************************************************************
sub hChar {
	my ($self, $str) = @_;

	if ($self->oka > 0) {
		$self->add2buf($str);
	} else {
		$self->print(encentis(dectext($str)));
	}
}

#**************************************************************************
sub hXMLDecl {
	my ($self, $ver, $enc, $stn) = @_;

	if ($stn) {
		$stn = ' standalone="yes"';
	} else {
		$stn = '';
	}

	if ($ver) {
		$ver = sprintf(' version="%s"', $ver);
	} else {
		$ver = '';
	}

	$self->printf("<?xml%s encoding=\"UTF-8\"%s?>", $ver, $stn);
}

#**************************************************************************
sub hDefault {
	my ($self, $str) = @_;

	$self->flushbuf;
	$self->print(dectext($str));
}

#******************************************************************
sub parse { 
	my ($self, $ifl, $ofl, $prg) = @_;

	my $ihn = openfile($ifl, 'r');
	if (defined($ihn)) {
		my $ohn = openfile($ofl, 'w');
		if (defined($ohn)) {

			my $enc = '';
			$ihn->read($enc, 1024);
			$ihn->seek(0, 0);
			if ($enc =~ m/<\?xml\s+[^>]+?encoding=(["'])(.*?)\1/) {
				$enc = $2;
			} else {
				$enc = '';
			}

			my $pere = $self->{PERE};
			$pere->{ihn} = $ihn;
			$pere->{ohn} = $ohn;
			$pere->{prg} = $prg;
			$pere->{buf} = '';
			$pere->{oka} = 0;

			$self->huk(1, '');
			$self->SUPER::parse($ihn, ProtocolEncoding => $enc || $self->enc);
			$self->huk(0, '');

			$ihn->close;
			$ohn->close;
			delete($pere->{ihn});
			delete($pere->{ohn});
			delete($pere->{prg});
		}
	}
}

#******************************************************************
sub new { 
	my ($class, $huk, $dic, $cas, $enc) = @_;

	my $self = $class->SUPER::new;

	$self->setHandlers(
		'Start'   => \&hStart,
    'End'     => \&hEnd,
    'Char'    => \&hChar,
		'XMLDecl' => \&hXMLDecl,
    'Default' => \&hDefault,
	);
	
	$self->{PERE} = {
		dic => $dic, 
		cas => $cas || 0, 
		enc => (defined($enc) && ($enc ne '')) ? $enc : "UTF-8",
		buf => '',
		oka => 0, 
		huk => $huk || sub { return 0}, 
	};

	return $self;
}

#******************************************************************
return 1;
__END__
